import Taro from '@tarojs/taro';

const ALIPAY = 'alipay';
class WebSocket {
    constructor(url) {
        this.url = url;
        this.readyState = this.CONNECTING;
        Taro.connectSocket({
            url
        })
            .then(ws => {
            this._ws = ws;
            ws.onClose(res => {
                this.readyState = this.CLOSED;
                this.onclose(res);
            });
            ws.onError(res => {
                this.readyState = this.CLOSED;
                this.onerror(res);
            });
            ws.onMessage(res => {
                this.onmessage(res);
            });
            if (this.readyState !== this.OPEN) {
                ws.onOpen(res => {
                    this.readyState = this.OPEN;
                    this.onopen(res);
                });
            }
            else {
                // 支付宝全局的 onSocketOpen 已触发过了，直接调用 onopen
                this.onopen({});
            }
        });
        // 支付宝只支持一个 socket 连接，且 onSocketOpen 的触发时机比 connectSocket 回调的时机早
        if (process.env.TARO_ENV === ALIPAY) {
            Taro.onSocketOpen(() => {
                this.readyState = this.OPEN;
            });
        }
    }
    send(data) {
        this._ws.send({
            data
        });
    }
    close(code, reason) {
        this.readyState = this.CLOSING;
        this._ws.close({
            code: code || 1000,
            reason: reason || ''
        });
    }
    get CONNECTING() {
        return 0;
    }
    get OPEN() {
        return 1;
    }
    get CLOSING() {
        return 2;
    }
    get CLOSED() {
        return 3;
    }
}

function createSocket(host) {
    class Socket {
        constructor() {
            this.isClosed = false;
            this.connect();
        }
        connect() {
            Taro.request({
                url: `http://${host}/socket.io/?EIO=3&transport=polling`
            })
                .then(({ data }) => {
                var _a;
                const target = ((_a = /{.*}/.exec(data)) === null || _a === void 0 ? void 0 : _a[0]) || '';
                try {
                    const res = JSON.parse(target);
                    this.sid = res.sid;
                    this.ackTimeout = res.pingTimeout;
                    this.initWS();
                }
                catch (error) {
                    console.error('Vue DevTools 握手失败: ', error);
                }
            })
                .catch(() => {
                console.warn('请打开 Vue DevTools');
            });
        }
        on(event, cb) {
            this[`on-${event}`] = cb;
        }
        emit(event, data) {
            if (this.isClosed)
                return;
            // heart break
            clearTimeout(this.ack);
            this.ack = setTimeout(() => this._ws.send('2'), 5000);
            const msgList = data ? [event, data] : [event];
            const msg = JSON.stringify(msgList);
            // message: 4 + parser.EVENT: 2
            this._ws.send('42' + msg);
        }
        initWS() {
            const ws = new WebSocket(`ws://${host}/socket.io/?EIO=3&transport=websocket&sid=${this.sid}`);
            ws.onopen = () => {
                // close: 1
                // message: 4
                // noop: 6
                // open: 0
                // ping: 2
                // pong: 3
                // upgrade: 5
                ws.send('2probe');
            };
            ws.onmessage = ({ data }) => {
                if (data === '3probe') {
                    // upgrade: 5
                    ws.send('5');
                    this['on-connect']();
                }
                else if (data === '3') {
                    // pong
                    this.ack = setTimeout(() => ws.send('2'), 5000);
                }
                else {
                    const match = /\[.*\]/.exec(data);
                    if (!match)
                        return;
                    try {
                        const [event, args = []] = JSON.parse(match[0]);
                        const cb = this[`on-${event}`];
                        if (typeof cb === 'function') {
                            // @TODO: 元素高亮暂时不实现，需要魔改 backend 的代码
                            const _args = args.filter(arg => arg.event.indexOf(':mouse-over') === -1);
                            cb(_args);
                        }
                    }
                    catch (error) {
                        console.error('Vue DevTools 解析 socket message 失败: ', error);
                    }
                }
            };
            ws.onclose = reason => {
                this.isClosed = true;
                this['on-disconnect'](reason);
            };
            ws.onerror = err => console.error('[Taro Socket]onerror', err);
            this._ws = ws;
        }
        disconnect() {
            this.clean();
            if (!this.isClosed) {
                // message: 4 + close: 1
                this._ws.send('41');
                this._ws.close();
            }
        }
        clean() {
            clearTimeout(this.ack);
        }
    }
    return new Socket();
}

const { connect } = require('./backend');
connect(__VUE_DEVTOOLS_HOSTNAME__, __VUE_DEVTOOLS_PORT__, {
    io: createSocket
});
//# sourceMappingURL=runtime.js.map
